;--- Allgemeines -------------------------------------------------------

.equ    F_CPU           	= 16000000		; in Hz

;--- RS232 Schnittstelle -----------------------------------------------

.equ 	BAUD  				= 9600          ; Baudrate Debug Schittstelle

.equ    FIFO_LENGHT     	= 128
.equ	LINE_BUFFER_SIZE	= 64


;--- SPI Schnittstelle -------------------------------------------------

.equ	SPI_PORT			= PORTB			; TMC
.equ	SPI_SCK				= 5;0			; SCK
.equ	SPI_SDI				= 4;2			; SDO
.equ	SPI_SDO				= 3;1			; SDI
.equ	SPI_CS				= 2;3			; /CS

.equ	SPI_DATA_RATE		= 1	// 0= F_CPU/2 | 1= F_CPU/4 | 2= F_CPU/8 | 3= F_CPU/16 | 4= F_CPU/32 | 5= F_CPU/64 | 6= F_CPU/128

//address register Memory Mapped
.equ	mSPCR				= 0x4C
.equ	mSPSR				= 0x4D
.equ	mSPDR				= 0x4E


;--- Port I/O ----------------------------------------------------------

.equ	EN_PORT				= PORTD
.equ	EN_PIN				= 5

.equ	LD2_PORT			= PORTC
.equ	LD2_PIN				= 1

;--- Flags -------------------------------------------------------------

; GPIOR0
.equ    RETURN_VAL         	= 0			// Return-Value [0= false | 1= true]
.equ    TICK_30us         	= 1
.equ    DRV_CURRENT      	= 2
.equ    CURRENT_STEP        = 3
.equ    WARNING				= 4
.equ    ERROR				= 5
.equ	BLINK_fast			= 6
.equ	BLINK_slow			= 7

; GPIOR1
.equ    txt_string         	= 0			// incomming Text String [0= false | 1= true]


; DRVSTATUS
.equ    tmcSG      			= 4			// (status_rd +2)
.equ    tmcOT      			= 5			// (status_rd +2)
.equ    tmcOTPW         	= 6			// (status_rd +2)
.equ    tmcS2SA      		= 7			// (status_rd +2)
.equ    tmcS2GB      		= 0			// (status_rd +1)
.equ    tmcOLA      		= 1			// (status_rd +1)
.equ    tmcOLB      		= 2			// (status_rd +1)
.equ    tmcSTST		        = 3			// (status_rd +1)

;--- Konstanten --------------------------------------------------------

.equ    CYCLES_PER_US   	= ((F_CPU)/1000000)
.equ    CYCLES_PER_MS   	= ((F_CPU)/1000)
.equ	t_TICK_VAL			= 30				// [s] 1/(F_CPU/(8*60))
.equ	t_BLINK_VAL			= 125				// [ms] 1/(F_CPU/(8*60))
.equ	step_CUR_UP			= 64				// ticksteps to dec tmc current
.equ	t_CURRENT_UP		= 1000				// [ms]

;--- lower Register ----------------------------------------------------
.def    iSREG       = r0
.def	NULL		= r1			; NULL
.def	SEUP		= r2			; NULL
.def	HYSTCUP_L	= r3			; NULL
.def	HYSTCUP_H	= r4			; NULL
.def	blink_cnt	= r5


;--- higher Register ---------------------------------------------------
.def    itemp_A     = r22
.def    itemp_B     = r23



;--- Macros ------------------------------------------------------------

.macro  load_p                      ; ldt 16Bit @1 nach @0 High und @0 Low
        ldi     @0H,HIGH(@1)
        ldi     @0L,LOW(@1)
.endmacro

;--- Berechnungen ------------------------------------------------------
.equ	TICK_VAL 		= (F_CPU*t_TICK_VAL)/(8*1000000)
;             		      [MHz] *    [s]   / ^Prescaller * 1000000
.equ	BLINK_VAL 		= (F_CPU*t_BLINK_VAL)/(1024*1000)
;             		      [MHz] *    [s]    / ^Prescaller * 1000
.equ	CURRENT_UP_VAL	= (t_CURRENT_UP*1000)/t_TICK_VAL
.if	CURRENT_UP_VAL > 65535
	.error "t_CURRENT_UP to large!"
.endif


;--- Internes Ram ------------------------------------------------------
.DSEG
.ORG    SRAM_START							; 128 Byte allgemein

;Name:				.byte 2			; Sensitivity coefficient
rx_read:              		.BYTE 2
rx_write:             		.BYTE 2
rx_buffer:            		.BYTE FIFO_LENGHT+3

line:						.BYTE LINE_BUFFER_SIZE
char_counter:				.BYTE 1

tx_buffer:          		.BYTE 20
parameter:					.BYTE 1
value_ascii:				.BYTE 52
value:						.BYTE 3
curdrive:					.BYTE 1
curhold:					.BYTE 1
tmc_current:				.BYTE 3
cur_cnt:					.BYTE 1
seup_cnt:					.BYTE 1
cur_up_cnt:					.BYTE 2
status_position:			.BYTE 3
status_stallguard:			.BYTE 3
status_sg_sm_cur:			.BYTE 3
status_rd:					.BYTE 3
tst_mask:					.BYTE 1
buffer:						.BYTE 40
