; Berechnungen
.equ UBRR_VAL   = ((F_CPU+BAUD*8)/(BAUD*16)-1)  ; clever runden
.equ BAUD_REAL  = (F_CPU/(16*(UBRR_VAL+1)))     ; Reale Baudrate
.equ BAUD_ERROR = ((BAUD_REAL*1000)/BAUD-1000)  ; Fehler in Promille
 
.if ((BAUD_ERROR>10) || (BAUD_ERROR<-10))       ; max. +/-10 Promille Fehler
  .error "Systematischer Fehler der Baudrate grsser 1 Prozent und damit zu hoch!"
.endif

init_uart:
; init fifo buffer
        sts     (rx_buffer+0),NULL
		sts     (rx_buffer+1),NULL
		sts     (rx_buffer+2),NULL

;        load_p  X,(DATA_BUF)

; Baudrate einstellen
		ldi		r16, Byte2(UBRR_VAL)
		sts		UBRR0H, r16
		ldi		r16, Byte1(UBRR_VAL)
		sts		UBRR0L, r16
; Frame-Format: 8 Bit
		ldi		r16, (1<<UCSZ01)|(1<<UCSZ00)
		sts		UCSR0C, r16					; Asynchron 8N1
		ldi		r16, (1<<RXCIE0)|(1<<TXEN0)|(1<<RXEN0)	; UART TX | UART RX
		sts		UCSR0B, r16					; einschalten 
		ret

;------------------------------------------------------------------------------------------------------------------------
uart_send:
		push	r16
uart_send_loop:
		ld		r16,X+
		tst		r16
		breq	uart_send_end
		rcall	uart_send_char
		rjmp	uart_send_loop
uart_send_end:
;		rcall	uart_send_enter
		pop 	r16
		ret

;------------------------------------------------------------------------------------------------------------------------
uart_fsend:
		push	r16
uart_fsend_loop:
		lpm		r16,Z+
		tst		r16
		breq	uart_fsend_end
		rcall	uart_send_char
		rjmp	uart_fsend_loop
uart_fsend_end:
;		rcall	uart_send_enter
		pop		r16
		ret

;------------------------------------------------------------------------------------------------------------------------
uart_send_char:
	push	r17
uart_send_char_wait:
	lds		r17,UCSR0A
	sbrs    r17, UDRE0				; Warten bis UDR zum senden bereit ist
	rjmp	uart_send_char_wait
    sts		UDR0, r16
	pop		r17
	ret								; und wieder zurck

;------------------------------------------------------------------------------------------------------------------------
uart_send_enter:
	ldi		r16, '\r'
	rcall	uart_send_char
	ldi		r16, '\n'
	rcall	uart_send_char
	ret


;------------------------------------------------------------------------------------------------------------------------
serial_read:
        cli
        load_p  Y,(rx_buffer)     ;Y = Pointer to FIFO
        ld      r16, Y		;Ret:  r16 = Read data, Z=1:FIFO empty
        cpi     r16, 0
        breq    PC+12
        dec     r16
        st      Y, r16
        ldd     r16, Y+1
        inc     r16
        std     Y+1, r16
        dec     r16
        andi    r16, FIFO_LENGHT -1
        add     YL, r16
        adc     YH,NULL
        ori     r16, 1
        ldd     r16, Y+3
		sei
        ret


;------------------------------------------------------------------------------------------------------------------------
USART_RXC:
    in      iSREG,SREG
	push    YL
	push    YH
	lds     itemp_A,UDR0
    load_p  Y,(rx_buffer)     ;Y = Pointer to FIFO

    ld      itemp_B,Y		;      r16 = Data to be stored
    cpi     itemp_B, FIFO_LENGHT -1	;Ret:  Z=1:FIFO full (not stored)
    breq    PC+12
    inc     itemp_B
    st      Y,itemp_B
    ldd     itemp_B,Y+2    ; write pointer
    inc     itemp_B
    std     Y+2,itemp_B
    dec     itemp_B
    andi    itemp_B, FIFO_LENGHT -1
    add     YL,itemp_B
    adc     YH,NULL
    ori     itemp_B,1
    std     Y+3,itemp_A

usart_rxc_end:
    pop     YH
	pop     YL
	out     SREG,iSREG
	reti


;------------------------------------------------------------------------------------------------------------------------

// copy string from FLASH to RAM with stringend '0'
copy_fstring:	;Source:Z		Destination:Y
	push	r20
copy_fstring_loop:
	lpm		r20, Z+
	tst		r20
	breq	copy_fstring_end
	st		Y+, r20
	rjmp	copy_fstring_loop
copy_fstring_end:
	st		Y+, r20
	pop		r20
	ret

;------------------------------------------------------------------------------------------------------------------------
// copy string from FLASH to RAM without stringend '0'
fstring_copy:	;Source:Z		Destination:Y
	push	r20
fstring_copy_loop:
	lpm		r20, Z+
	tst		r20
	breq	fstring_copy_end
	st		Y+, r20
	rjmp	fstring_copy_loop
fstring_copy_end:
	pop		r20
	ret
